import {Component, createRef} from 'react';

// backspace starts the editor on Windows
const KEY_BACKSPACE = 'Backspace';
const KEY_DELETE = 'Delete';
const KEY_ENTER = 'Enter';
const KEY_TAB = 'Tab';

//Reference: https://www.ag-grid.com/react-data-grid/cell-editors/
export default class NumericEditor extends Component {
    constructor(props) {
        super(props);

        this.inputRef = createRef(null);

        const isCharacter = this.props.eventKey && this.props.eventKey.length === 1;

        this.cancelBeforeStart = isCharacter && '1234567890'.indexOf(this.props.eventKey) < 0;
        this.state = this.createInitialState(props);
        this.onKeyDown = this.onKeyDown.bind(this);
        this.handleChange = this.handleChange.bind(this);
    }

    componentDidMount() {
        this.setCaret();
    }

    render() {
        return (
            <input
                ref={this.inputRef}
                className="numericAgGridField"
                data-testid="numeric-editor-input"
                style={{width: '100%'}}
                value={this.state.value}
                onKeyDown={this.onKeyDown}
                onChange={this.handleChange}
            />
        );
    }

    /* Component Editor Lifecycle methods */

    // the final value to send to the grid, on completion of editing
    getValue() {
        return this.state.value;
    }

    // Gets called once before editing starts, to give editor a chance to
    // cancel the editing before it even starts.
    isCancelBeforeStart() {
        return this.cancelBeforeStart;
    }

    // Gets called once when editing is finished (eg if Enter is pressed).
    // If you return true, then the result of the edit will be ignored.
    isCancelAfterEnd() {
        // will reject the number if it greater than 1,000,000
        // not very practical, but demonstrates the method.
        return this.state.value > 1000000;
    }

    /* Utility methods */
    createInitialState(props) {
        let startValue;

        if (props.eventKey === KEY_BACKSPACE || props.eventKey === KEY_DELETE) {
            startValue = '';
        } else if (props.eventKey && props.eventKey.length === 1) {
            startValue = props.eventKey;
        } else {
            startValue = props.value;
        }

        return {
            value: startValue,
        };
    }

    onKeyDown(event) {
        if (this.isLeftOrRight(event) || this.isBackspace(event) || this.isDelete(event)) {
            event.stopPropagation();
            return;
        }

        if (!this.finishedEditingPressed(event) && !this.isNumericKey(event)) {
            if (event.preventDefault) {
                event.preventDefault();
            }
        }
    }

    isLeftOrRight(event) {
        return ['ArrowLeft', 'ArrowRight'].indexOf(event.key) > -1;
    }

    handleChange(event) {
        this.setState({value: event.target.value});
    }

    getCharCodeFromEvent(event) {
        return event?.which || event?.keyCode
    }

    isCharNumericOrDot(charStr) {
        return !Number.isNaN(Number.parseInt(charStr, 10)) || charStr === '.'
    }

    isNumericKey(event) {
        const charCode = this.getCharCodeFromEvent(event);
        const charStr = event.key ? event.key : String.fromCharCode(charCode);
        //this check will ensure that you can only enter valid (decimal) numbers as a whole
        const containsOneOrZeroDots = `${event.target.value + charStr}`.split('.').length <= 2;
        return containsOneOrZeroDots && this.isCharNumericOrDot(charStr);
    }


    isBackspace(event) {
        return event.key === KEY_BACKSPACE;
    }

    isDelete(event) {
        return event.key === KEY_DELETE;
    }

    finishedEditingPressed(event) {
        const key = event.key;
        return key === KEY_ENTER || key === KEY_TAB;
    }

    setCaret() {
        setTimeout(() => {
            const currentInput = this.inputRef.current;
            currentInput.focus();
        });
    }
}