import React, { Component } from 'react';
import { Redirect } from 'react-router-dom';
import _ from 'lodash';
import { publish } from '../../App';
import moment from 'moment';
import { Toast } from 'primereact/toast';
import { Dropdown } from 'primereact/dropdown';
import { InputText } from 'primereact/inputtext';
import { InputTextarea } from 'primereact/inputtextarea';
import Flatpickr from "react-flatpickr";
import AppLoader from '../../layout/components/AppLoader';
import PageHeader from '../../layout/components/PageHeader';
import UIConstants from '../../utils/ui.constants';
import { CustomDialog } from '../../layout/components/CustomDialog';
import { InputMask } from 'primereact/inputmask';

import ProjectService from '../../services/project.service';
import ReservationService from '../../services/reservation.service';
import Jeditor from '../../components/JSONEditor/JEditor';
import UtilService from '../../services/util.service';
import UnitConverter from '../../utils/unit.converter';
import { appGrowl } from '../../layout/components/AppGrowl';
import FormActionbar from '../../components/FormActionbar'; 
import "flatpickr/dist/flatpickr.css";
import ParserUtility from '../../utils/parser.utility';

/**
 * Component to create a new Reservation
 */
export class ReservationCreate extends Component {
    constructor(props) {
        super(props);
        this.state = {
            showDialog: false,
            isDirty: false,
            isLoading: true,
            redirect: null,
            paramsSchema: null,                     // JSON Schema to be generated from strategy template to pass to JSON editor 
            dialog: { header: '', detail: '' },      // Dialog properties
            touched: {
                name: '',
            },
            reservation: {
                name: '',
                description: '',
                start_time: null,
                stop_time: null,
                duration: null,
                project: (props.match ? props.match.params.project : null) || null,
            },
            reservationStrategy: {
                id: null,
            },
            errors: {},                             // Validation Errors
            validFields: {},                        // For Validation
            validForm: false,                       // To enable Save Button
            validEditor: false,
            createAnother: false,
            multiCreateCount: 1
        };
        this.projects = [];                         // All projects to load project dropdown
        this.reservationTemplates = [];
        this.reservationStrategies = [];

        // Validateion Rules
        this.formRules = {
            name: { required: true, message: "Name can not be empty" },
            description: { required: true, message: "Description can not be empty" },
            // project: {required: true, message: "Project can not be empty"},
            start_time: { required: true, message: "Start Time can not be empty" },
        };
        this.tooltipOptions = UIConstants.tooltipOptions;
        this.setEditorOutput = this.setEditorOutput.bind(this);
        this.saveReservation = this.saveReservation.bind(this);
        this.reset = this.reset.bind(this);
        this.cancelCreate = this.cancelCreate.bind(this);
        this.checkIsDirty = this.checkIsDirty.bind(this);
        this.close = this.close.bind(this);
        this.initReservation = this.initReservation.bind(this);
        this.changeStrategy = this.changeStrategy.bind(this);
        this.setEditorFunction = this.setEditorFunction.bind(this);
        this.isValidDuration = this.isValidDuration.bind(this);
        this.setCreateAnother = this.setCreateAnother.bind(this);
    }

    async componentDidMount() {
        await this.initReservation();
    }

    /**
     * Initialize the reservation and relevant details
     */
    async initReservation() {
        const promises = [ProjectService.getProjectList(),
        ReservationService.getReservationTemplates(),
        UtilService.getUTC(),
        ReservationService.getReservationStrategyTemplates()
        ];
        let emptyProjects = [{ url: null, name: "Select Project" }];
        Promise.all(promises).then(responses => {
            this.projects = emptyProjects.concat(responses[0]);
            this.reservationTemplates = responses[1];
            let systemTime = moment.utc(responses[2]);
            this.reservationStrategies = responses[3];
            let reservationTemplate = this.reservationTemplates.length>0?this.reservationTemplates[0]:null;
            let schema = {
                properties: {}
            };
            if (reservationTemplate) {
                schema = reservationTemplate.schema;
                schema = ParserUtility.updateReservationPropsOrder(schema, true);
            }
            this.setState({
                paramsSchema: _.cloneDeep(schema),
                isLoading: false,
                reservationTemplate: reservationTemplate,
                systemTime: systemTime,
            });
        });
    }

    /**
     * 
     * @param {Id} strategyId - id value of reservation strategy template
     */
    async changeStrategy(strategyId) {
        this.setState({ isLoading: true });
        const reservationStrategy = _.find(this.reservationStrategies, { 'id': strategyId });
        const reservationTemplate = _.find(this.reservationTemplates, { 'id': reservationStrategy.reservation_template_id});
        let paramsOutput = {};
        if (reservationStrategy.template.parameters) {
            //if reservation strategy has parameter then prepare output parameter

        } else {
            paramsOutput = _.cloneDeep(reservationStrategy.template);
            delete paramsOutput["$id"];
        }
        let reservation = this.state.reservation;
        if (!this.nameEdited || reservation.name === "") {
            reservation.name = reservationStrategy.name;
        }
        if (!this.descEdited || reservation.description === "") {
            reservation.description = reservationStrategy.template.activity.description;
        }
        let schema = {
            properties: {}
        };
        if (reservationTemplate) {
            schema = reservationTemplate.schema;
            schema = ParserUtility.updateReservationPropsOrder(schema, true);
        }
        this.setState({
            isLoading: false,
            reservation: reservation,
            reservationTemplate: reservationTemplate,
            reservationStrategy: reservationStrategy,
            paramsSchema: _.cloneDeep(schema),
            paramsOutput: paramsOutput,
            isDirty: true
        });
        if (this.state.editorFunction) {
            if (this.state.reservationTemplate === reservationTemplate) {
                this.state.editorFunction("setValue", paramsOutput);
            }   else {
                this.state.editorFunction();
            }
        }
    }

    /**
     * Function to set form values to the Reservation object
     * @param {string} key 
     * @param {object} value 
     */
    setReservationParams(key, value) {
        let reservation = _.cloneDeep(this.state.reservation);
        reservation[key] = value;
        if (!this.state.isDirty && !_.isEqual(this.state.reservation, reservation)) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({
                reservation: reservation, validForm: this.validateForm(key), validEditor: this.validateEditor(), touched: {
                    ...this.state.touched,
                    [key]: true
                }, isDirty: true
            });
        } else {
            this.setState({
                reservation: reservation, validForm: this.validateForm(key), validEditor: this.validateEditor(), touched: {
                    ...this.state.touched,
                    [key]: true
                }
            });
        }
    }    

    /**
    * This function is mainly added for Unit Tests. If this function is removed Unit Tests will fail.
    */
    validateEditor() {
        return this.validEditor;
    }

    /**
     * Function to call on change and blur events from input components
     * @param {string} key 
     * @param {any} value 
     */
    setParams(key, value, type) {
        let reservation = this.state.reservation;
        switch (type) {
            case 'NUMBER': {
                reservation[key] = value ? parseInt(value) : 0;
                break;
            }
            default: {
                reservation[key] = value;
                break;
            }
        }
        if(!this.state.isDirty) {
            publish('edit-dirty', true);
        }
        this.setState({ reservation: reservation, validForm: this.validateForm(key), isDirty: true },
            () => {
                this.setDurationOrEndValue(key);
            });
    }

    /**
     * Validation function to validate the form or field based on the form rules.
     * If no argument passed for fieldName, validates all fields in the form.
     * @param {string} fieldName 
     */
    validateForm(fieldName) {
        let validForm = false;
        let errors = this.state.errors;
        let validFields = this.state.validFields;
        if (fieldName) {
            delete errors[fieldName];
            delete validFields[fieldName];
            if (this.formRules[fieldName]) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.reservation[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message ? rule.message : `${fieldName} is required`;
                    } else {
                        validFields[fieldName] = true;
                    }
                }
            }
        } else {
            errors = {};
            validFields = {};
            for (const fieldName in this.formRules) {
                const rule = this.formRules[fieldName];
                const fieldValue = this.state.reservation[fieldName];
                if (rule.required) {
                    if (!fieldValue) {
                        errors[fieldName] = rule.message ? rule.message : `${fieldName} is required`;
                    } else {
                        validFields[fieldName] = true;
                    }
                }
            }
        }
        this.setState({ errors: errors, validFields: validFields });
        if (Object.keys(validFields).length === Object.keys(this.formRules).length) {
            validForm = true;
            delete errors['start_time'];
            delete errors['stop_time'];
        }
        if (!this.validateDates(this.state.reservation.start_time, this.state.reservation.stop_time)) {
            validForm = false;
            if (!fieldName || fieldName === 'start_time') {
                errors['start_time'] = "Start Time cannot be same or after End Time";
                delete errors['stop_time'];
            }
            if (!fieldName || fieldName === 'stop_time') {
                errors['stop_time'] = "End Time cannot be same or before Start Time";
                delete errors['start_time'];
            }
            this.setState({ errors: errors });
        }
        if (fieldName === 'duration' && this.state.reservation.duration) {
            var values = this.state.reservation.duration.split(' ');
            var days = values[0];
            var dValues = values[1].split(':');           
            delete errors['duration'];
            if ((days *1 )===0 && (dValues[0] * 1) === 0 && (dValues[1] * 1) === 0 && (dValues[2] * 1) === 0) {
                validForm = false;
                if (!fieldName || fieldName === 'duration') {
                    errors['duration'] = "Duration cannot be zero";
                }
                this.setState({ errors: errors });
            }
        }
        return validForm;
    }

    /**
     * Function to validate if stop_time is always later than start_time if exists.
     * @param {Date} fromDate 
     * @param {Date} toDate 
     * @returns boolean
     */
    validateDates(fromDate, toDate) {
        if (fromDate && toDate && moment(toDate).isSameOrBefore(moment(fromDate))) {
            return false;
        }
        return true;
    }

    setEditorOutput(jsonOutput, errors) {
        this.paramsOutput = jsonOutput;
        this.validEditor = errors.length === 0;
        if (!this.state.isDirty && this.state.paramsOutput && !_.isEqual(this.state.paramsOutput, jsonOutput)) {
            if(!this.state.isDirty) {
                publish('edit-dirty', true);
            }
            this.setState({
                paramsOutput: jsonOutput,
                validEditor: errors.length === 0,
                validForm: this.validateForm(),
                isDirty: true
            });
        } else {
            this.setState({
                paramsOutput: jsonOutput,
                validEditor: errors.length === 0,
                validForm: this.validateForm()
            });
        }
    }

    async saveReservation() {
        let reservation = this.state.reservation;
        let project = this.projects.find(project => project.name === reservation.project);
        reservation['start_time'] = moment(reservation['start_time']).format(UIConstants.CALENDAR_DATETIME_FORMAT);
        reservation['stop_time'] = reservation['stop_time'] ? moment(reservation['stop_time']).format(UIConstants.CALENDAR_DATETIME_FORMAT) : null;
        reservation['project'] = project ? project.url : null;
        reservation['specifications_template'] = this.state.reservationTemplate.url;
        reservation['specifications_doc'] = this.paramsOutput;
        reservation = await ReservationService.saveReservation(reservation);
        if (reservation && reservation.id) {
            appGrowl.show({severity: 'success', summary: 'Success', detail: 'Reservation is created successfully.'});
            //const dialog = { header: 'Success', detail: 'Reservation is created successfully. Do you want to create another Reservation?' };
            if (this.state.createAnother) { 
                this.setState({paramsOutput: {}, showDialog: false, isDirty: false });
                this.reset();
            }   else {
                this.setState({paramsOutput: {}, showDialog: false, isDirty: false, redirect: '/reservation/list' });
            }
            
            publish('edit-dirty', false);
        }/*  else {
            this.growl.show({severity: 'error', summary: 'Error Occured', detail: 'Unable to save Reservation', showDialog: false, isDirty: false});
        }*/
    }

    /**
     * Reset function to be called when user wants to create new Reservation
     */
    reset() {
        let tmpReservation = {
            name: '',
            description: '',
            start_time: '',
            stop_time: '',
            project: '',
        }
        this.nameEdited = false;
        this.descEdited = false;
        let multiCreateCount = this.state.multiCreateCount;
        this.setState({
            dialogVisible: false,
            dialog: { header: '', detail: '' },
            errors: [],
            reservation: tmpReservation,
            reservationStrategy: {
                id: null,
            },
            paramsSchema: null,
            paramsOutput: null,
            validEditor: false,
            validFields: {},
            touched: false,
            stationGroup: [],
            showDialog: false,
            isDirty: false,
            multiCreateCount: ++multiCreateCount
        });
        this.initReservation();
    }

    /**
   * Cancel Reservation creation and redirect
   */
    cancelCreate() {       
        publish('edit-dirty', false);
        if (this.state.multiCreateCount > 1) {
            this.props.history.go(this.state.multiCreateCount*-1);
        }   else {
            this.props.history.length>1?this.state.isDirty?this.props.history.go(-2):this.props.history.go(-1):this.props.history.push(`/su/timelineview/week`);
        }
        this.setState({ showDialog: false });
    }

    /**
     * warn before cancel the page if any changes detected 
     */
    checkIsDirty() {
        if (this.state.isDirty) {
            this.setState({ showDialog: true });
        } else {
            this.cancelCreate();
        }
    }

    close() {
        this.setState({ showDialog: false });
    }

    /**
     * JEditor's function that to be called when parent wants to trigger change in the JSON Editor
     * @param {Function} editorFunction 
     */
    setEditorFunction(editorFunction) {
        this.setState({ editorFunction: editorFunction });
    }

    /**
     * Function to set the value for the dependant fields when value is set to one field.
     * When start_time or stop_time is changed, duration will be updated accordingly.
     * Similarly if duration is changed, stop_time is updated.
     * @param {String} key - property name of the reservation
     */
    setDurationOrEndValue = (key) => {
        //let state = this.state;
        if (key === 'start_time' || key === 'stop_time') {
            if (this.state.reservation.start_time && this.state.reservation.stop_time) {              
                var delta = Math.abs(this.state.reservation.start_time - this.state.reservation.stop_time) / 1000;
                let tempDuration = UnitConverter.getSecsToDDHHmmss(delta);             
                this.setDurationOrStopTime('duration', tempDuration);
            }   else if (key === 'start_time' && this.state.reservation.start_time && this.state.reservation.duration) {
                let stopDate = UnitConverter.getEndDateFromDuration(this.state.reservation.start_time, this.state.reservation.duration);               
                this.setDurationOrStopTime('stop_time', stopDate);
            }   else if (key === 'stop_time' && !this.state.reservation.stop_time) {
                this.setDurationOrStopTime('duration', "");
            }
        }
        else if (key === 'duration') {
            if (this.state.reservation.start_time && this.state.reservation.duration) {   
                let stopDate = UnitConverter.getEndDateFromDuration(this.state.reservation.start_time, this.state.reservation.duration);               
                this.setDurationOrStopTime('stop_time', stopDate);
            }
        }
    }

    /**
     * Function to set calcualted value for either duration or stop_time
     * @param {String} key - name of the field
     * @param {*} value - value to set for the field
     */
    setDurationOrStopTime(key, value) {
        let reservation = this.state.reservation;
        reservation[key] = value;
        this.setState({ reservation: reservation, validForm: this.validateForm(key), isDirty: true });
    }

    /**
     * Function to validate the duration field.
     * @param {String} value - Duration in format 'Days HH:mm:ss'
     * @returns boolean
     */
    isValidDuration(value) {
        let errors = this.state.errors;
        let touched = this.state.touched;
        let reservation = this.state.reservation;
        let validForm = this.state.validForm;
        if (value.length === 12 && (value === "000 00:00:00" ||
            !value.match(/^([0-1]90 00:00:00)|([0-1][0-8][0-9] ([0-1]?\d|2[0-3]):([0-5]?\d):([0-5]?\d))$/))) {
            errors.duration = "Not valid duration. Duration should be in Days Hours:minutes:seconds. Min - 000 00:00:01, Max - 190 00:00:00";
            touched.duration = true;
            validForm = false;
        }   else {
            delete errors["duration"];
            delete touched["duration"];
            validForm = this.validateForm();
        }
        reservation.duration = value;
        this.setState({errors: errors, touched: touched, reservation: reservation, validForm: validForm});
        return errors.duration?false:true;
    }

    /**
     * Select/Unselect the create another check box
     * @param {*} e 
     */
     setCreateAnother(e) {
        this.setState({'createAnother': e.target.checked})
    }

    render() {
        if (this.state.redirect) {
            return <Redirect to={{ pathname: this.state.redirect }}></Redirect>
        }
        const schema = this.state.paramsSchema;
        let jeditor = null;
        if (schema) {
            if (this.state.reservation.specifications_doc) {
                delete this.state.reservation.specifications_doc.$id;
                delete this.state.reservation.specifications_doc.$schema;
            }
            jeditor = React.createElement(Jeditor, {
                title: "Reservation Parameters",
                schema: schema,
                initValue: this.state.paramsOutput,
                callback: this.setEditorOutput,
                parentFunction: this.setEditorFunction
            });
        }
        return (
            <React.Fragment>
                <Toast ref={(el) => this.growl = el} />
                <PageHeader location={this.props.location} title={'Reservation - Add'}
                    actions={[{
                        icon: 'fa-window-close', title: 'Click to close Reservation creation',
                        type: 'button', actOn: 'click', props: { callback: this.checkIsDirty }
                    }]} />
                { this.state.isLoading ? <AppLoader /> :
                    <>
                        <div>
                            <div className="p-fluid">
                            <div className="p-field p-grid">
                            <label htmlFor="strategy" className="col-lg-2 col-md-2 col-sm-12">Reservation Strategy</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12" data-testid="strategy" >
                                        <Dropdown inputId="strategy" optionLabel="name" optionValue="id"
                                            tooltip="Choose Reservation Strategy Template to set default values for create Reservation" tooltipOptions={this.tooltipOptions}
                                            value={this.state.reservationStrategy.id}
                                            options={this.reservationStrategies}
                                            onChange={(e) => { this.changeStrategy(e.value) }}
                                            placeholder="Select Strategy" />
                                        <label className={(this.state.errors.reservationStrategy && this.state.touched.reservationStrategy) ? "error" : "info"}>
                                            {(this.state.errors.reservationStrategy && this.state.touched.reservationStrategy) ? this.state.errors.reservationStrategy : "Select Reservation Strategy Template"}
                                        </label>
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label htmlFor="project" className="col-lg-2 col-md-2 col-sm-12">Project</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12" data-testid="project" >
                                        <Dropdown inputId="project" optionLabel="name" optionValue="name"
                                            tooltip="Project" tooltipOptions={this.tooltipOptions}
                                            value={this.state.reservation.project}
                                            options={this.projects}
                                            onChange={(e) => { this.setParams('project', e.value) }}
                                            placeholder="Select Project" />
                                        <label className={(this.state.errors.project && this.state.touched.project) ? "error" : "info"}>
                                            {(this.state.errors.project && this.state.touched.project) ? this.state.errors.project : "Select Project"}
                                        </label>
                                    </div>
                                </div>
                                <div className="p-field p-grid">
                                    <label htmlFor="reservationname" className="col-lg-2 col-md-2 col-sm-12">Name <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <InputText className={(this.state.errors.name && this.state.touched.name) ? 'input-error' : ''} id="reservationname" data-testid="name"
                                            tooltip="Enter name of the Reservation Name" tooltipOptions={this.tooltipOptions} maxLength="128"
                                            ref={input => { this.nameInput = input; }}
                                            value={this.state.reservation.name} autoFocus
                                            onChange={(e) => {this.setReservationParams('name', e.target.value); this.nameEdited=true}}
                                            onBlur={(e) => this.setReservationParams('name', e.target.value)} />
                                        <label className={(this.state.errors.name && this.state.touched.name) ? "error" : "info"}>
                                            {this.state.errors.name && this.state.touched.name ? this.state.errors.name : "Max 128 characters"}
                                        </label>
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label htmlFor="description" className="col-lg-2 col-md-2 col-sm-12">Description <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <InputTextarea className={(this.state.errors.description && this.state.touched.description) ? 'input-error' : ''} rows={3} cols={30}
                                            tooltip="Longer description of the Reservation"
                                            tooltipOptions={this.tooltipOptions}
                                            maxLength="128"
                                            data-testid="description"
                                            value={this.state.reservation.description}
                                            onChange={(e) => {this.setReservationParams('description', e.target.value); this.nameEdited=true}}
                                            onBlur={(e) => this.setReservationParams('description', e.target.value)} />
                                        <label className={(this.state.errors.description && this.state.touched.description) ? "error" : "info"}>
                                            {(this.state.errors.description && this.state.touched.description) ? this.state.errors.description : "Max 255 characters"}
                                        </label>
                                    </div>
                                </div>

                                <div className="p-field p-grid">
                                    <label className="col-lg-2 col-md-2 col-sm-12">Start Time <span style={{ color: 'red' }}>*</span></label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <Flatpickr data-enable-time data-input options={{
                                            "inlineHideInput": true,
                                            "wrap": true,
                                            "enableSeconds": true,
                                            "time_24hr": true,
                                            "minuteIncrement": 1,
                                            "allowInput": true,
                                            "defaultDate": this.state.systemTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT),
                                            "defaultHour": this.state.systemTime.hours(),
                                            "defaultMinute": this.state.systemTime.minutes()
                                        }}
                                            title="Start of this reservation"
                                            value={this.state.reservation.start_time}
                                            onChange={value => {
                                                this.setParams('start_time', value[0] ? value[0] : this.state.reservation.start_time);
                                                this.setReservationParams('start_time', value[0] ? value[0] : this.state.reservation.start_time)
                                            }} >
                                            <input type="text" data-input className={`p-inputtext p-component ${this.state.errors.start_time && this.state.touched.start_time ? 'input-error' : ''}`} />
                                            <i className="fa fa-calendar" data-toggle style={{ position: "absolute", marginLeft: '-25px', marginTop: '5px', cursor: 'pointer' }} ></i>
                                            <i className="pi pi-times" style={{ position: "absolute", marginLeft: '-50px', marginTop: '5px', cursor: 'pointer' }}
                                                onClick={() => { this.setParams('start_time', ''); this.setReservationParams('start_time', '') }}></i>
                                        </Flatpickr>
                                        <label className={this.state.errors.start_time && this.state.touched.start_time ? "error" : "info"}>
                                            {this.state.errors.start_time && this.state.touched.start_time ? this.state.errors.start_time : ""}
                                        </label>
                                    </div>
                                    <div className="col-lg-1 col-md-1 col-sm-12"></div>
                                    <label className="col-lg-2 col-md-2 col-sm-12">End Time</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <Flatpickr data-enable-time data-input options={{
                                            "inlineHideInput": true,
                                            "wrap": true,
                                            "enableSeconds": true,
                                            "time_24hr": true,
                                            "minuteIncrement": 1,
                                            "allowInput": true,
                                            "minDate": this.state.reservation.start_time ? this.state.reservation.start_time.toDate : '',
                                            "defaultDate": this.state.systemTime.format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT),
                                            "defaultHour": this.state.systemTime.hours(),
                                            "defaultMinute": this.state.systemTime.minutes()
                                        }}
                                            title="End of this reservation. If empty, then this reservation is indefinite."
                                            value={this.state.reservation.stop_time}
                                            onChange={value => {
                                                this.setParams('stop_time', value[0] ? value[0] : this.state.reservation.stop_time);
                                                this.setReservationParams('stop_time', value[0] ? value[0] : this.state.reservation.stop_time)
                                            }} >
                                            <input type="text" data-input className={`p-inputtext p-component ${this.state.errors.stop_time && this.state.touched.stop_time ? 'input-error' : ''}`} />
                                            <i className="fa fa-calendar" data-toggle style={{ position: "absolute", marginLeft: '-25px', marginTop: '5px', cursor: 'pointer' }} ></i>
                                            <i className="pi pi-times" style={{ position: "absolute", marginLeft: '-50px', marginTop: '5px', cursor: 'pointer' }}
                                                onClick={() => { this.setParams('stop_time', ''); this.setReservationParams('stop_time', '') }}></i>
                                        </Flatpickr>
                                        <label className={this.state.errors.stop_time && this.state.touched.stop_time ? "error" : "info"}>
                                            {this.state.errors.stop_time && this.state.touched.stop_time ? this.state.errors.stop_time : ""}
                                        </label>
                                    </div>
                                </div>

                                <div className="p-field p-grid">
                                    <label className="col-lg-2 col-md-2 col-sm-12">Duration</label>
                                    <div className="col-lg-3 col-md-3 col-sm-12">
                                        <InputMask mask="999 99:99:99"
                                            tooltip="Enter duration in (Days HH:mm:ss) format. Min-000 00:00:01 & Max-190 00:00:00."
                                            tooltipOptions={this.tooltipOptions} 
                                            value={this.state.reservation.duration}
                                            placeholder="DDD HH:mm:ss"
                                            onChange={(e) =>{if(!e.value) {this.setDurationOrEndValue("stop_time")}}}
                                            onComplete={(e) => {
                                                if (this.isValidDuration(e.value)) {
                                                    this.setReservationParams('duration', e.value);
                                                    this.setParams('duration', e.value ? e.value : this.state.reservation.duration);
                                                }
                                            }}
                                            onBlur={(e) => {
                                                if (e.value) {
                                                    this.setParams('duration', e.value);
                                                 } else {
                                                    this.setDurationOrEndValue("stop_time")
                                                 }
                                            }}></InputMask>
                                        <label className={this.state.errors.duration && this.state.touched.duration ? "error" : "info"}>
                                            {this.state.errors.duration && this.state.touched.duration ? this.state.errors.duration : ""}
                                        </label>
                                    </div>
                                </div>

                                <div className="p-grid">
                                    <div className="p-col-12" style={{width: "100%"}}>
                                        {this.state.paramsSchema ? jeditor : ""}
                                    </div>
                                </div>
                            </div> 
                        </div>
                    <div className="p-breadcrumb footer-long-div actions" >
                        <FormActionbar createAnotherCallBack={this.setCreateAnother} createAnother={this.state.createAnother} 
                        tooltip="Select checkbox to create another reservation after saving this reservation"
                        onSubmit={this.saveReservation} disableSaveBtn={!this.state.validEditor || !this.state.validForm} onCancel={this.checkIsDirty} />
                    </div>
                    </>
                }

                {/* Dialog component to show messages and get input */}
                <div className="p-grid" data-testid="confirm_dialog">
                    <CustomDialog type="confirmation" visible={this.state.showDialog} width="40vw"
                        header={'Add Reservation'} message={'Do you want to discard your changes? A new Reservation will not be created.'}
                        content={''} onClose={this.close} 
                        actions={ [ {id:"yes", title: 'Discard', callback: this.cancelCreate, className:'act-btn-dispose'},
                        {id:"no", title: 'Cancel', className:'act-btn-cancel', callback: this.close} ]}  >
                    </CustomDialog>
                </div>
            </React.Fragment>
        );
    }
}

export default ReservationCreate;