import { SchedulingUnitListLite } from "../Scheduling/SchedulingUnitListLite.js"
import { getDefaultGrouping, getGroups } from "./business/timelinegroupingbusiness.js";
import { expandTimeLineRowsForGrouping } from "./business/timelineRowBusiness.js";
import Filters from "./components/toolbar/Filters";
import { useEffect, useRef, useState } from "react";
import 'react-calendar-timeline/lib/Timeline.css';
import "flatpickr/dist/flatpickr.css";
import PeriodSelection from "../../shared/periodSelection"
import Timeline, {
    CursorMarker,
    CustomMarker,
    DateHeader,
    SidebarHeader,
    TimelineHeaders,
    TimelineMarkers
} from "react-calendar-timeline";
import moment from "moment";
import { getIntervalRendererLST } from "./helpers/timeline.headers.helper";
import UIConstants from "../../utils/ui.constants";
import { groupRenderer, itemRenderer } from "./helpers/timeline.renderer.helper";
import containerResizeDetector from "react-calendar-timeline/lib/resize-detector/container";
import TimelineItemPopover from "./components/TimelineItemPopover";
import {
    getCursorDateTimes,
    getHeaderActions,
    getPageHeaderOptionsMenuItems,
    getSchedulerActions,
    getGroupedByActions,
    updateSchedulerStatuses,
} from "./helpers/week.view.helper";
import { getStore, updateStore } from "../../services/store.helper";
import DateTimeNavigator from "./components/toolbar/DateTimeNavigator";
import ZoomAndMove from "./components/toolbar/ZoomAndMove";
import { ProgressBar } from "primereact/progressbar";
import Legendbar from "./components/Legendbar";
import SchedulingUnitSummary from "../Scheduling/summary";
import { useLocation } from "react-router-dom";
import { editConstraintSettings, updateSUConstraintTemplate } from "../Scheduling/summary.helper";
import PageHeader from "../../layout/components/PageHeader";
import { CustomDialog } from "../../layout/components/CustomDialog";
import DynamicScheduler from "../../components/DynamicScheduler";
import { TieredMenu } from "primereact/tieredmenu";

import { ProgressSpinner } from 'primereact/progressspinner';
import ReservationSummary from "../Reservation/reservation.summary";
import {
    fetchLSTShift, fetchReservationSummaryInformation,
    fetchSchedulerStatuses,
    fetchSunTimings, fetchSUSummaryInformation,
    fetchTimelineData,
    fetchUserPermissions,
} from "./data/week.view.data";



import useWeekViewWebSocket from "../../utils/websocket.js";

function getTimelineHeaders(headerSettings) {
    return <TimelineHeaders className="sticky">
        <SidebarHeader>
            {({ getRootProps }) => { //getRootProps is mandatory for the library
                return <div {...getRootProps()} className="sidebar-header">
                    <div className="sidebar-header-row">UTC</div>
                    <div className="sidebar-header-row" title="LST@CS002" >LST</div>
                </div>
            }}
        </SidebarHeader>
        <DateHeader unit={headerSettings.unit} labelFormat="HH:mm" />
        <DateHeader unit={headerSettings.unit} intervalRenderer={({ getIntervalProps, intervalContext }) => {
            return getIntervalRendererLST({ getIntervalProps, intervalContext }, headerSettings.lstShiftInSeconds)
        }} />
    </TimelineHeaders>;
}


let DelayedTooltip;
function refreshCursorData(date, setCursorTimeUTC) {
    try {
        if (DelayedTooltip) {
            clearTimeout(DelayedTooltip);
        }
    } catch (e) { /* empty */ }

    DelayedTooltip = setTimeout(() => { setCursorTimeUTC(date) }, 0);
}


function getTimelineMarkers(setCursorDate) {
    const now = moment(moment().utc(false).format(UIConstants.UTC_DATE_TIME_FORMAT))
    return <TimelineMarkers>
        <CustomMarker date={now.valueOf()}>
            {({ styles }) => {
                const customStyles = {
                    ...styles,
                    backgroundColor: 'var(--green)',
                    zIndex: 998
                }
                return <div style={customStyles} />
            }}
        </CustomMarker>
        <CursorMarker>
            {({ styles, date }) => {
                refreshCursorData(date, setCursorDate);
                const customStyles = {
                    ...styles,
                    backgroundColor: 'var(--blue-400)',
                    zIndex: 998,
                    height: "100%"
                }
                return <div style={customStyles} />
            }}
        </CursorMarker>
    </TimelineMarkers>;
}

/**
 * Parent component. Check data flow and actions here: https://drive.google.com/file/d/11iZOwfD4fG7TC-24POEId_TVkNczE0O3/view?pli=1
 * Also, please keep the diagram up-to-date
 * @return {JSX.Element}
 * @constructor
 */
export default function WeekView(props) {
    const timelineStore = getStore(UIConstants.STORE_KEY_TIMELINE)
    const location = useLocation()
    let actionsMenuRef = useRef()
    const isDebugLoggingOn = false
    //general page info
    const [permissions, setPermissions] = useState()
    const [schedulerSettings, setSchedulerSettings] = useState({
        isDynamicSchedulerOn: undefined,
        isFixedTimeSchedulerOn: undefined,
        isDynamicSchedulerActive: undefined,
    })
    const [showSchedulerSettingsDialog, setShowSchedulerSettingsDialog] = useState(false)

    const [grouping, setGrouping] = useState(timelineStore.grouping ?? getDefaultGrouping())

    //data info
    const [isLoading, setIsLoading] = useState(false)
    const [refetchToggle, setRefetchToggle] = useState(false)
    const [isExpanded, setIsExpanded] = useState(true)
    const [timelineClassName, setTimelineClassName] = useState("timeline-expanded-filter")
    const [tabelViewAdjustment, setTabelViewAdjustment] = useState(20)
    const [data, setData] = useState({
        schedulingUnits: [],
        reservations: [],
        noReFiltering: false
    })
    const [keys, setKeys] = useState(
        {
            groupIdKey: 'id',
            groupTitleKey: 'title',
            groupRightTitleKey: 'rightTitle',
            itemIdKey: 'id',
            itemTitleKey: 'title',    // key for item div content
            itemDivTitleKey: 'title', // key for item div title (<div title="text"/>)
            itemGroupKey: grouping,
            itemTimeStartKey: 'start_time',
            itemTimeEndKey: 'end_time',
        }
    )
    //timeline component

    const [weekNumbers, setWeekNumbers] = useState("");
    const [startTime, setStartTime] = useState(props.startTime ? props.startTime : moment().startOf('day'))
    const [endTime, setEndTime] = useState(props.endTime ? props.endTime : moment().endOf('day'))

    const [items, setItems] = useState({
        schedulingUnitItems: [],
        plainFilterSchedulingUnitItems: [],
        schedulingAdditionalGroupingItems: [],
        reservationAdditionalGroupingItems: [],
        reservationItems: [],
        sunTimingsItems: []
    })

    const [groups, setGroups] = useState([])
    const [headerSettings, setHeaderSettings] = useState({
        timeSteps: timelineStore.timeSteps ?? UIConstants.ALL_TIMESTEPS[0],
        unit: timelineStore.headerUnit ?? "hour",
        lstShiftInSeconds: 0
    })
    const [visibleTime, setVisibleTime] = useState({
        start: undefined,
        end: undefined
    })

    //timeline item pop-over component
    const [mouseOverItem, setMouseOverItem] = useState()
    const [popPosition, setPopPosition] = useState({ display: 'none' })
    const [summaryItem, setSummaryItem] = useState()

    //marker that also displays time information on thge group
    const [cursorTimeUTC, setCursorTimeUTC] = useState()
    const [cursorInformation, setCursorInformation] = useState({
        utc: undefined,
        lst: undefined,
        stationlst: undefined
    })

    const [showLegendbar, setShowLegendbar] = useState(false)
    const [showTableView, setShowTableView] = useState(false)
    const [showSummary, setShowSummary] = useState(false)



    //details summary
    const [summarySettings, setSummarySettings] = useState({
        isLoading: false,
        schedulingUnitItem: undefined,
        reservationItem: undefined,
        summaryConstraintTemplate: undefined
    })
    //TODO: needed for SU summary component; should be refactored into that component + updater functions
    const [onClose, setOnClose] = useState(false)
    const [cancelStatus, setCancelStatus] = useState(false)
    const [onSaveDisabled, setOnSaveDisabled] = useState(true)


    useEffect(() => {
        if (isExpanded) {
            setTimelineClassName("timeline-expanded-filter")
            setTabelViewAdjustment({ height: 375 })
        } else {
            setTimelineClassName("timeline-colapsed-filter")
            setTabelViewAdjustment({ height: 130 })
        }

    }, [isExpanded])

    useEffect(
        () => {
            async function GetDateToolTipForMouseOver() {
                if (groups == undefined) return;
                const currentGroupIndex = groups.findIndex(group => group.id === mouseOverItem?.[keys.itemGroupKey]);
                if (currentGroupIndex >= 0) {
                    let group = groups[currentGroupIndex];
                    const currentGroupDate = group.date;
                    if (isDebugLoggingOn) console.log(" mouseOverItem[" + keys.itemGroupKey + "]=" + mouseOverItem?.[keys.itemGroupKey] + "date:" + currentGroupDate, "isSingleStation:" + group.isSingleStation, groups[currentGroupIndex])
                    await getCursorDateTimes(cursorTimeUTC, currentGroupIndex, currentGroupDate, headerSettings.lstShiftInSeconds, group.designatedStation, group.isSingleStation, setCursorInformation);
                    let newGroups = groups.map(group => ({ ...group }));
                    newGroups.forEach(group => delete group.cursorInfo)
                    newGroups[currentGroupIndex].cursorInfo = cursorInformation
                    setGroups(newGroups)
                }
            }
            GetDateToolTipForMouseOver();
        }, [cursorTimeUTC])


    useEffect(() => { //initializer

        const currentTimelinestore = getStore(UIConstants.STORE_KEY_TIMELINE)

        const shouldFetchReservations = currentTimelinestore.reservationsToggle;

        setIsLoading(true)

        if (!Object.values(visibleTime).some(value => value === undefined)) { //TODO: extract
            const startDate = moment(startTime).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT)
            const visibleStartTime = moment(visibleTime.start).format(UIConstants.CALENDAR_TIME_FORMAT)
            const visibleEndTime = moment(visibleTime.end).format(UIConstants.CALENDAR_TIME_FORMAT)
            const startDateWithVisibleStartTime = moment(startDate + " " + visibleStartTime, UIConstants.CALENDAR_DATETIME_FORMAT);
            const startDateWithVisibleEndTime = moment(startDate + " " + visibleEndTime, UIConstants.CALENDAR_DATETIME_FORMAT);
            setVisibleTime({
                start: startDateWithVisibleStartTime,
                end: startDateWithVisibleEndTime
            })
        }
        setWeekString(startTime, endTime);
        const loadingStates = []
        loadingStates.push(fetchTimelineData(startTime, endTime, setData, shouldFetchReservations).catch(e => console.error("Couldn't get scheduling units and/or reservations", e)))
        loadingStates.push(fetchLSTShift(startTime, setHeaderSettings).catch(e => console.error("Couldn't get current lst from the server", e)))
        if (!permissions) {
            loadingStates.push(fetchUserPermissions(setPermissions).catch(e => console.error("Couldn't retrieve the user permissions:", e)))
        }
        if (Object.values(schedulerSettings).some(value => value === undefined)) {
            loadingStates.push(fetchSchedulerStatuses(setSchedulerSettings).catch(e => console.error("Couldn't retrieve the scheduler's statuses (dynamic on, fixed time on, dynamic active:", e)))
        }
        Promise.all(loadingStates).then(() => {

            setIsLoading(false)
        });
    }, [startTime, endTime, refetchToggle])


    useEffect(() => { //summary information
        if (summaryItem === undefined) {
            setShowSummary(false)
            setSummarySettings({
                isLoading: false,
                schedulingUnitItem: undefined,
                reservationItem: undefined,
                summaryConstraintTemplate: undefined
            })
        } else {
            setSummarySettings(prevState => ({
                ...prevState,
                isLoading: true,
            }))
            setShowSummary(true)
            if (summaryItem?.type === "RESERVATION") {
                fetchReservationSummaryInformation(summaryItem?.id, setSummarySettings).catch(e => console.error("Couldn't retrieve reservations details for id: ", summaryItem?.id, e))
            } else { //a Scheduling Unit summary is selected
                fetchSUSummaryInformation(summaryItem?.id, setSummarySettings, summaryItem?.item).catch(e => console.error("Couldn't retrieve summary details for id: ", summaryItem?.id, e))
            }
        }
    }, [summaryItem]);

    function setWeekString(newStart, newEnd) {
        let weekString = newStart.week() === newEnd.week() ? newStart.week() : newStart.week() + "," + newEnd.week();
        weekString = "Week " + weekString;
        setWeekNumbers(weekString);
    }

    function navigateToWeek(addingWeeks) {
        const newStart = moment(startTime).startOf('day').add(addingWeeks, 'weeks')
        setStartTime(newStart);
        const newEnd = moment(endTime).endOf('day').add(addingWeeks, 'weeks')
        setEndTime(newEnd);
        setWeekString(newStart, newEnd);

    }

    // websocket handling


    async function refreshSunsettings(currentGroups = groups, itemGroupKey = keys.itemGroupKey) {
        await fetchSunTimings(startTime, setItems, currentGroups, itemGroupKey);
    }




    useEffect(() => {
        // Check if the forceUpdate prop has changed and trigger a re-render
        if (props.grouping !== undefined)
            updateGrouping(props.grouping);
    }, [props.grouping]);

    //  when the data update, has updatet the items, we have all the groups in expect station grouping
    // this is due to the fact that this is a LARGE amount of data
    function refreshGrouping(grouper = grouping, schedulingUnitItems = items.schedulingUnitItems, reserVationUnitItems = items.reservationItems) {
        const groupsplittedItemsScheduling = expandTimeLineRowsForGrouping(schedulingUnitItems, grouper);
        const groupsplittedItemsReservations = expandTimeLineRowsForGrouping(reserVationUnitItems, grouper);
        let combinedItems = [...schedulingUnitItems, ...groupsplittedItemsScheduling, ...groupsplittedItemsReservations]
        let groupupdates = getGroups(grouper, startTime, combinedItems, endTime);
        setGroups(groupupdates.groups);

        refreshSunsettings(groupupdates.groups, groupupdates.itemGroupKey);

        if (groupsplittedItemsScheduling?.length > 0) {
            setItems(prevData => ({
                ...prevData,
                schedulingAdditionalGroupingItems: groupsplittedItemsScheduling
            }));
        }


        setItems(prevData => ({
            ...prevData,
            reservationAdditionalGroupingItems: groupsplittedItemsReservations
        }));



    }

    function updateGrouping(updatedGrouping) {
        if (grouping == updatedGrouping) {
            return;
        }
        //  Setting Grouping from .. to
        setKeys(prevData => ({ ...prevData, itemGroupKey: updatedGrouping }));
        updateStore(UIConstants.STORE_KEY_TIMELINE, timelineStore, {
            ["grouping"]: updatedGrouping,
        });

        setGrouping(updatedGrouping);
        refreshGrouping(updatedGrouping);
    }

    // websocket hook that opens and allows interaction via the wss connection
    // todo: there is probably a better way to access the component state from the hook?
    useWeekViewWebSocket(data, setData, summaryItem, setSummaryItem, setShowSummary, startTime, endTime);

    /**
   * Toggles Legend, and when not expanded , expand
   * @param {boolean} newState - State of Legend
   */

    function toggleLegend(newState) {
        if (newState && !isExpanded) {
            setIsExpanded(true);
        }
        setShowLegendbar(newState);
    }


    if (isDebugLoggingOn) console.log("Render TimeLine grouping on :", keys.itemGroupKey, items.schedulingUnitItems, items.schedulingAdditionalGroupingItems, groups);

    return <div>
        <div className={(isLoading ? "fix-element" : "hide-element")}>
            <ProgressBar className={isLoading ? "" : "hide-element"} mode="indeterminate"
                style={{ height: '0.25rem' }} />
        </div>
        {!permissions ? <ProgressSpinner className="m-4" style={{ color: "var(--primary-300)" }} /> :
            <TieredMenu className="app-header-menu" popup ref={actionsMenuRef}
                model={getPageHeaderOptionsMenuItems(permissions, startTime, endTime)} />}
        <PageHeader location={location} title={'Scheduling Units - ' + weekNumbers + ' View'}
            actions={getGroupedByActions(grouping, updateGrouping).concat(PeriodSelection.getPeriodNavigationActions(navigateToWeek)).concat(getSchedulerActions(schedulerSettings, setShowSchedulerSettingsDialog))
                .concat(getHeaderActions(isLoading, setRefetchToggle, actionsMenuRef, isExpanded, setIsExpanded, showLegendbar, toggleLegend, showTableView, setShowTableView))} />
        <div className="p-grid">


            <div className="timeline-panel">

                <div className="timeline-tools" style={{ display: isExpanded ? 'flex' : 'none' }}>

                    <Filters timelineStore={timelineStore}
                        visibleStartTime={visibleTime.start}
                        endTime={endTime}
                        startTime={startTime}
                        setItems={setItems}
                        data={data}
                        setRefetchToggle={setRefetchToggle}
                        refreshGrouping={refreshGrouping}
                    />
                    <DateTimeNavigator timelineStore={timelineStore}
                        startTime={startTime}
                        setStartTime={setStartTime}
                        setEndTime={setEndTime}
                    />
                    <ZoomAndMove timelineStore={timelineStore}
                        startTime={startTime}
                        setVisibleTime={setVisibleTime}
                        visibleTime={visibleTime}
                        headerSettings={headerSettings}
                        setHeaderSettings={setHeaderSettings}
                    />

                    <Legendbar className={showLegendbar ? undefined : "hide-element"} />

                </div>


                {showTableView && <div className={timelineClassName} > <SchedulingUnitListLite adjust={tabelViewAdjustment} blueprints={items?.plainFilteredSchedulingUnitItems}></SchedulingUnitListLite></div>}
                {!showTableView && (
                    Object.values(visibleTime).some(value => value === undefined) ? <ProgressSpinner /> :
                        <Timeline
                            className={timelineClassName}
                            groups={groups}
                            keys={keys}
                            items={[...items.sunTimingsItems, ...items.schedulingUnitItems, ...items.schedulingAdditionalGroupingItems, ...items.reservationItems, ...items.reservationAdditionalGroupingItems]}
                            defaultTimeStart={startTime}
                            defaultTimeEnd={endTime}
                            visibleTimeStart={visibleTime.start.valueOf()}
                            visibleTimeEnd={visibleTime.end.valueOf()}
                            timeSteps={headerSettings.timeSteps.value}
                            buffer={1}
                            //general styling settings timeline
                            lineHeight={40}
                            itemHeightRatio={1}
                            resizeDetector={containerResizeDetector}
                            //general action settings timeline
                            canMove={false}
                            canResize={false}
                            stackItems={true}
                            //render settings
                            groupRenderer={({ group }) => groupRenderer({ group })}
                            itemRenderer={({
                                item,
                                itemContext,
                                getItemProps
                            }) => itemRenderer({
                                item,
                                itemContext,
                                getItemProps
                            }, setPopPosition, setMouseOverItem, setSummaryItem)}
                        >
                            {getTimelineHeaders(headerSettings)}
                            {getTimelineMarkers(setCursorTimeUTC)}
                        </Timeline>)}
            </div>
            {showSummary && (
                <div className="timeline-panel summary">
                    {showSummary &&
                        (summarySettings.isLoading ? <ProgressSpinner className="m-4" style={{ color: "var(--primary-300)" }} />
                            : (summaryItem?.type === "RESERVATION" ?
                                (summarySettings.reservationItem === undefined ?
                                    <ProgressSpinner className="m-4" style={{ color: "var(--primary-300)" }} /> :
                                    <ReservationSummary reservation={summarySettings.reservationItem}
                                        viewInNewWindow
                                        location={location}
                                        closeCallback={() => setShowSummary(false)}>
                                    </ReservationSummary>) :
                                (summarySettings.schedulingUnitItem === undefined ?
                                    <ProgressSpinner className="m-4" style={{ color: "var(--primary-300)" }} /> :
                                    <SchedulingUnitSummary
                                        schedulingUnit={summarySettings.schedulingUnitItem}
                                        viewInNewWindow
                                        constraintsTemplate={summarySettings.summaryConstraintTemplate}
                                        onUpdate={(suId, updatedConstraintTemplate) =>
                                            updateSUConstraintTemplate(suId, updatedConstraintTemplate, setOnClose, setOnSaveDisabled, setShowSummary)}
                                        newPopupvalue={() => {
                                        }}
                                        onClose={onClose}
                                        onSavedisable={onSaveDisabled}
                                        onCloseFn={(val) => editConstraintSettings(val, setOnClose, setOnSaveDisabled, setCancelStatus)}
                                        closeCallback={() => setShowSummary(false)}
                                        cancelCheckstatus={(val) => setCancelStatus(val)}
                                        cancelStatus={cancelStatus}
                                        location={location}>
                                    </SchedulingUnitSummary>))

                        )}
                </div>)}
        </div>
        {mouseOverItem ?
            <div className="p-overlaypanel p-component timeline-popover"
                style={{ ...popPosition }}>
                <TimelineItemPopover mouseOverItem={mouseOverItem} />
            </div> : null}
        <div className="p-grid" data-testid="confirm_dialog">
            <CustomDialog type="success"
                visible={showSchedulerSettingsDialog}
                showIcon={false}
                header="Scheduling Settings"
                message={<DynamicScheduler callBack={async (val, status) =>
                    await updateSchedulerStatuses(val, status, setSchedulerSettings)}
                />}
                onClose={() => setShowSchedulerSettingsDialog(false)}
            />
        </div>
    </div>
}


