import moment from "moment/moment";
import UIConstants from "../../../../utils/ui.constants";

/**
 * Determines whether zooming in/out is possible based on the available zoomLevels
 * @param zoomLevelIndex current zoom level its index
 * @return {{canZoomIn: boolean, canZoomOut: boolean}}
 */
export function getZoomPossibilities(zoomLevelIndex) {
    let zoomPossibilities = {}

    if (zoomLevelIndex > -1) { //only for valid zoom levels enable the zoom possibilities
        if (zoomLevelIndex >= UIConstants.ALL_ZOOM_LEVELS_WEEK.length - 1) { //largest zoom level reached
            zoomPossibilities.canZoomIn = true
            zoomPossibilities.canZoomOut = false
            return zoomPossibilities
        } else if (zoomLevelIndex === 0) { //smallest zoom level reached
            zoomPossibilities.canZoomIn = false
            zoomPossibilities.canZoomOut = true
            return zoomPossibilities
        } else {
            zoomPossibilities.canZoomIn = true
            zoomPossibilities.canZoomOut = true
            return zoomPossibilities
        }
    }

    zoomPossibilities.canZoomIn = false
    zoomPossibilities.canZoomOut = false
    return zoomPossibilities
}

/**
 * Determines whether moving left/right is possible based on the visible start/end time
 * @param visibleTime {{start, end}} times that is visible w.r.t the timeline
 * @return {{moveLeft: boolean, moveRight: boolean}}
 */
export function getMovePossibilities(visibleTime) {
    let movePossibilities = {}
    movePossibilities.moveLeft = false
    movePossibilities.moveRight = false

    if (visibleTime) { //only for valid times, enable the move possibilities
        const minimumVisibleStartTime = moment(moment(visibleTime.start).startOf('day').format(UIConstants.CALENDAR_DATETIME_FORMAT));
        if (moment(visibleTime.start).isAfter(minimumVisibleStartTime)) {
            movePossibilities.moveLeft = true
        }
        const maximumVisibleEndTime = moment(moment(visibleTime.start).endOf('day').format(UIConstants.CALENDAR_DATETIME_FORMAT));
        if (moment(visibleTime.end).isBefore(maximumVisibleEndTime)) {
            movePossibilities.moveRight = true
        }
    }

    return movePossibilities
}

/**
 * Sets the new visible start/end times by adding or subtracting the 'minuteAmount'.
 * It takes into account the extrema the start (00:00:00) and end times (23:59:99) can have
 * @param setVisibleTime state callback function for times shown w.r.t. the timeline
 * @param minuteAmount number (negative means subtract, positive means add)
 */
export function moveTimeline(setVisibleTime, minuteAmount) {
    setVisibleTime(previousVisibleTime => {
        let newStartTime = moment(previousVisibleTime.start).add(minuteAmount, 'minutes');
        let newEndTime = moment(previousVisibleTime.end).add(minuteAmount, 'minutes');
        return {
            start: getNewTimeOrExtrema(newStartTime, previousVisibleTime.start),
            end: getNewTimeOrExtrema(newEndTime, previousVisibleTime.start)
        }
    })
}

/**
 * Calculates the visible times based on the level's values
 * @param selectedZoomLevel  {name: string, days: number, hours: number, minutes: number}
 * @param selectedTime {moment}
 * @return {{start: moment, end: moment}}
 */
function getZoomTimesHoursMinutes(selectedZoomLevel, selectedTime) {
    let zoomTimes = {}
    if (selectedZoomLevel.hours === undefined || selectedZoomLevel.minutes === undefined) {
        console.error("Couldn't zoom the time line because the selectedZoomLevel is invalid:", selectedZoomLevel)
    } else {
        let newStartTime, newEndTime;
        ['hours', 'minutes'].forEach(timeKey => {
            if (selectedZoomLevel[timeKey] > 0) {
                const amount = selectedZoomLevel[timeKey] * 0.5;
                newStartTime = moment(newStartTime || selectedTime).subtract(amount, timeKey);
                newEndTime = moment(newEndTime || selectedTime).add(amount, timeKey);
            }
        })
        zoomTimes.start = getNewTimeOrExtrema(newStartTime, selectedTime)
        zoomTimes.end = getNewTimeOrExtrema(newEndTime, selectedTime)
    }
    return zoomTimes
}

/**
 * Transforms the selected zoom level into the visible start and end times based on the current time ('now')
 * If the zoomlevel is a 'day' value, it sets it to the extrema (00:00:00, 23:59:99)
 * If the zoom level has 'hours/minutes' then:
 * - start = now - (1/2 * hours/minutes)
 * - end = now + (1/2 * hours/minutes)
 * @param selectedZoomLevel {name: string, days: number, hours: number, minutes: number}
 * @param selectedTime
 * @param startDate the date start date of the view in the timeline
 * @return {{start: moment, end: moment}}
 */
export function getZoomTimes(selectedZoomLevel, selectedTime, startDate) {
    const currentTime = moment().utc().format(UIConstants.CALENDAR_TIME_FORMAT).split(":")
    return getZoomTimesOnTime(selectedZoomLevel, selectedTime, startDate, currentTime);
}


export function getZoomTimesOnTime(selectedZoomLevel, selectedTime, startDate, currentTime) {
    let newSelectedTime = moment(moment(startDate).set({ hour: currentTime[0], minutes: currentTime[1] })).format(UIConstants.CALENDAR_DATETIME_FORMAT)
    if (selectedTime !== undefined) {
        const selectedMoment = moment(moment(startDate).format(UIConstants.CALENDAR_DEFAULTDATE_FORMAT) + " " + selectedTime, UIConstants.CALENDAR_DATETIME_FORMAT);
        newSelectedTime = selectedMoment.isValid() ? selectedMoment : newSelectedTime
    }
    let zoomTimes = {}
    if (selectedZoomLevel.days > 0) {
        const minimumVisibleStartTime = moment(newSelectedTime).startOf('day').format(UIConstants.CALENDAR_DATETIME_FORMAT);
        zoomTimes.start = moment(minimumVisibleStartTime)
        const maximumVisibleEndTime = moment(newSelectedTime).endOf('day').format(UIConstants.CALENDAR_DATETIME_FORMAT);
        zoomTimes.end = moment(maximumVisibleEndTime)
    } else {
        zoomTimes = getZoomTimesHoursMinutes(selectedZoomLevel, newSelectedTime)
    }
    return zoomTimes;
}
/**
 * If the new start time is before the minimum time (00:00:00) return the minimum time
 * If the new end time is after the maximum time (23:59:99) it return the maximum time
 * @param newTime
 * @param visibleStartTime
 * @return {*|moment.Moment}
 */
export function getNewTimeOrExtrema(newTime, visibleStartTime) {
    const minimumVisibleStartTime = moment(visibleStartTime).startOf('day').format(UIConstants.CALENDAR_DATETIME_FORMAT);
    if (newTime.isBefore(moment(minimumVisibleStartTime))) {
        return moment(minimumVisibleStartTime)
    }
    const maximumVisibleEndTime = moment(visibleStartTime).endOf('day').format(UIConstants.CALENDAR_DATETIME_FORMAT);
    if (newTime.isAfter(moment(maximumVisibleEndTime))) {
        return moment(maximumVisibleEndTime)
    }
    return newTime
}

/**
 * The span and time steps are initially linked so that you have better view per header span.
 * @param selectedZoomLevel  {name: string, days: number, hours: number, minutes: number}
 * @return {*} time steps as wanted by the Timeline library
 */
export function getTimeStepsForZoom(selectedZoomLevel) {
    if (selectedZoomLevel.days > 0) {
        return UIConstants.ALL_TIMESTEPS[0] // Default to 1 hr
    }

    const timeStepsMap = {
        0: UIConstants.ALL_TIMESTEPS[UIConstants.ALL_TIMESTEPS.length - 1], // 1 min
        1: UIConstants.ALL_TIMESTEPS[UIConstants.ALL_TIMESTEPS.length - 2], // 5 min
        3: UIConstants.ALL_TIMESTEPS[UIConstants.ALL_TIMESTEPS.length - 3], // 10 min
        6: UIConstants.ALL_TIMESTEPS[UIConstants.ALL_TIMESTEPS.length - 4], // 30 min
    };

    return timeStepsMap[selectedZoomLevel.hours] || UIConstants.ALL_TIMESTEPS[0]; // Default to 1 hr
}