import {
    getMovePossibilities,
    getNewTimeOrExtrema, getTimeStepsForZoom,
    getZoomPossibilities,
    getZoomTimes,
    moveTimeline,
    getZoomTimesOnTime
} from "./zoomandmove.helper";
import moment from "moment";
import UIConstants from "../../../../utils/ui.constants";


describe('getNewTimeOrExtrema', () => {
    const testFormat = 'HH:mm:ss';
    const MIN_VISIBLE_START = moment('26-05-1992T00:00:00', testFormat);
    const MAX_VISIBLE_END = moment('26-05-1992-T23:59:59', testFormat);
    const selectedTime = moment("1992-05-26 00:00:00", UIConstants.CALENDAR_DATETIME_FORMAT)

    it('returns the extrema start of day if new time is before the start of day', () => {
        const newTime = MIN_VISIBLE_START.subtract(1, 'hours')
        const result = getNewTimeOrExtrema(newTime, selectedTime);
        expect(result).toBe(MIN_VISIBLE_START);
    });

    it('returns the extrema end of day if new time is after the end of day', () => {
        const newTime = MAX_VISIBLE_END.add(1, 'second');
        const result = getNewTimeOrExtrema(newTime, selectedTime);
        expect(result).toBe(MAX_VISIBLE_END);
    });

    test.each(['12:34:56', '00:00:00', '23:59:99'])('returns the new time if it is within the visible range: %s', (time) => {
        const date = "1992-05-26"
        const newTime = moment(date + " " + time);
        const result = getNewTimeOrExtrema(newTime, selectedTime);
        expect(result).toBe(newTime)
    })
});

describe('getZoomTimes', () => {
    const selectedTime = moment("1992-05-26 00:00:00", UIConstants.CALENDAR_DATETIME_FORMAT)
    const startDate = moment("2023-08-15 08:22:31", UIConstants.CALENDAR_DATETIME_FORMAT)
    const currentDate = moment("2023-08-15 02:00:00", UIConstants.CALENDAR_DATETIME_FORMAT)

    it('returns extrema times when selectedZoomLevel has days', () => {
        const selectedZoomLevel = { days: 1 };
        const result = getZoomTimes(selectedZoomLevel, selectedTime, startDate);
        expect(result.start.format('HH:mm:ss')).toEqual('00:00:00');
        expect(result.end.format('HH:mm:ss')).toEqual('23:59:59');
    });
    

    it('returns calculated times when selectedZoomLevel has minutes', () => {
        const selectedZoomLevel = { hours: 0, minutes: 30 };

        const result = getZoomTimesOnTime(selectedZoomLevel, selectedTime, startDate, currentDate);

        const expectedStart = '08:07:31';
        const expectedEnd = '08:37:31';

        expect(result.start.format('HH:mm:ss')).toEqual(expectedStart);
        expect(result.end.format('HH:mm:ss')).toEqual(expectedEnd);
    });
    
    it('returns calculated times when selectedZoomLevel has hours', () => {
        const selectedZoomLevel = { hours: 3, minutes: 0 };

        const result = getZoomTimesOnTime(selectedZoomLevel, selectedTime, startDate, currentDate);

        const expectedStart = '06:52:31';
        const expectedEnd = '09:52:31';

        expect(result.start.format('HH:mm:ss')).toEqual(expectedStart);
        expect(result.end.format('HH:mm:ss')).toEqual(expectedEnd);
    });

    it('returns calculated times when selectedZoomLevel has hours and minutes', () => {
        const selectedZoomLevel = { name: "hi", days: 0, hours: 3, minutes: 30 };

        const result = getZoomTimesOnTime(selectedZoomLevel, selectedTime, startDate, currentDate);

        const expectedStart = '06:37:31';
        const expectedEnd = '10:07:31';

        expect(result.start.format('HH:mm:ss')).toEqual(expectedStart);
        expect(result.end.format('HH:mm:ss')).toEqual(expectedEnd);
    });

    it('handles invalid selectedZoomLevel', () => {
        const selectedZoomLevel = { hours: 2 }; // Missing 'minutes'
        console.error = jest.fn();

        const result = getZoomTimes(selectedZoomLevel, selectedTime, startDate);

        expect(result).toEqual({});
        expect(console.error).toHaveBeenCalledWith(
            "Couldn't zoom the time line because the selectedZoomLevel is invalid:",
            selectedZoomLevel
        );
    });
});

describe('moveTimeline', () => {
    let dateNowSpy
    const mockVisibleTime = {
        start: moment('10:00:00', 'HH:mm:ss'),
        end: moment('12:00:00', 'HH:mm:ss')
    }

    const setVisibleTime = jest.fn(callback => callback(mockVisibleTime));

    beforeEach(() => {
        //needs to be a timestamp in milliseconds for moment methods to work
        //1692087751000 ===  Tuesday, August 15, 2023 8:22:31
        const timeStamp = 1692087751000
        dateNowSpy = jest.spyOn(Date, 'now').mockImplementation(() => timeStamp);
    });

    afterEach(() => {
        dateNowSpy.mockRestore();
    });

    it('moves timeline forward by adding 30 minutes', () => {
        moveTimeline(setVisibleTime, 30);

        expect(setVisibleTime).toHaveBeenCalledWith(expect.any(Function));

        const newVisibleTimeUpdater = setVisibleTime.mock.calls[0][0]
        const newVisibleTime = newVisibleTimeUpdater(mockVisibleTime)

        expect(newVisibleTime.start.format('HH:mm:ss')).toEqual('10:30:00');
        expect(moment(newVisibleTime.end).format('HH:mm:ss')).toEqual('12:30:00');
    });


    it('moves timeline backward by subtracting 15 minutes', () => {
        moveTimeline(setVisibleTime, -15);

        expect(setVisibleTime).toHaveBeenCalledWith(expect.any(Function));

        const newVisibleTimeUpdater = setVisibleTime.mock.calls[0][0]
        const newVisibleTime = newVisibleTimeUpdater(mockVisibleTime)

        expect(newVisibleTime.start.format('HH:mm:ss')).toEqual('09:45:00');
        expect(newVisibleTime.end.format('HH:mm:ss')).toEqual('11:45:00');
    });
});

describe('getMovePossibilities', () => {
    it('returns move possibilities as false for invalid times', () => {
        const movePossibilities = getMovePossibilities(null);
        expect(movePossibilities).toEqual({ moveLeft: false, moveRight: false });
    });

    it('returns move possibilities as true when both times are within bounds', () => {
        const visibleTime = {
            start: moment('10:00:00', 'HH:mm:ss'),
            end: moment('15:00:00', 'HH:mm:ss')
        }

        const movePossibilities = getMovePossibilities(visibleTime);
        expect(movePossibilities).toEqual({ moveLeft: true, moveRight: true });
    });

    it('returns move left as false when selectedTime is at the minimum', () => {
        const visibleTime = {
            start: moment('00:00:00', 'HH:mm:ss'),
            end: moment('15:00:00', 'HH:mm:ss')
        }

        const movePossibilities = getMovePossibilities(visibleTime);
        expect(movePossibilities).toEqual({ moveLeft: false, moveRight: true });
    });

    it('returns move right as false when visibleEndTime is at the maximum', () => {
        const visibleTime = {
            start: moment('10:00:00', 'HH:mm:ss'),
            end: moment('23:59:59', 'HH:mm:ss')
        }

        const movePossibilities = getMovePossibilities(visibleTime);
        expect(movePossibilities).toEqual({ moveLeft: true, moveRight: false });
    });

    it('returns move possibilities as false when both times are at extrema', () => {
        const visibleTime = {
            start: moment('00:00:00', 'HH:mm:ss'),
            end: moment('23:59:59', 'HH:mm:ss')
        }

        const movePossibilities = getMovePossibilities(visibleTime);
        expect(movePossibilities).toEqual({ moveLeft: false, moveRight: false });
    });
});

describe('getZoomPossibilities', () => {
    it('returns zoom possibilities as false for invalid zoomLevelIndex', () => {
        const zoomLevelIndex = -1;
        const zoomPossibilities = getZoomPossibilities(zoomLevelIndex);
        expect(zoomPossibilities).toEqual({ canZoomIn: false, canZoomOut: false });
    });

    it('returns zoom out as false when zoomLevelIndex is at largest zoom level', () => {
        const zoomLevelIndex = UIConstants.ALL_ZOOM_LEVELS_WEEK.length - 1;
        const zoomPossibilities = getZoomPossibilities(zoomLevelIndex);
        expect(zoomPossibilities).toEqual({ canZoomIn: true, canZoomOut: false });
    });

    it('returns zoom in as false when zoomLevelIndex is at smallest zoom level', () => {
        const zoomLevelIndex = 0;
        const zoomPossibilities = getZoomPossibilities(zoomLevelIndex);
        expect(zoomPossibilities).toEqual({ canZoomIn: false, canZoomOut: true });
    });

    it('returns zoom possibilities when zoomLevelIndex is within bounds', () => {
        const zoomLevelIndex = 2;
        const zoomPossibilities = getZoomPossibilities(zoomLevelIndex);
        expect(zoomPossibilities).toEqual({ canZoomIn: true, canZoomOut: true });
    });
});

describe('getTimeStepsForZoom', () => {
    it('should return 1 min time step for 30 min', () => {
        const selectedZoomLevel = { days: 0, hours: 0, minutes: 30 };
        const result = getTimeStepsForZoom(selectedZoomLevel);
        expect(result.value).toEqual({ hour: 0, minute: 1 });
    });

    it('should return 5 min time step for 1 hour', () => {
        const selectedZoomLevel = { days: 0, hours: 1, minutes: 0 };
        const result = getTimeStepsForZoom(selectedZoomLevel);
        expect(result.value).toEqual({ hour: 0, minute: 5 });
    });

    it('should return 10 min time step for 3 hours', () => {
        const selectedZoomLevel = { days: 0, hours: 3, minutes: 0 };
        const result = getTimeStepsForZoom(selectedZoomLevel);
        expect(result.value).toEqual({ hour: 0, minute: 10 });
    });

    it('should return 30 min time step for 6 hours', () => {
        const selectedZoomLevel = { days: 0, hours: 6, minutes: 0 };
        const result = getTimeStepsForZoom(selectedZoomLevel);
        expect(result.value).toEqual({ hour: 0, minute: 30 });
    });
    it('should default to 1 hr time step for other values', () => {
        const selectedZoomLevel = { days: 0, hours: 3.14, minutes: 1337 };
        const result = getTimeStepsForZoom(selectedZoomLevel);
        expect(result.value).toEqual({ hour: 1, minute: 0 });
    });
    it('should default to 1 hr time step for days', () => {
        const selectedZoomLevel = { days: 1, hours: 0, minute: 0 };
        const result = getTimeStepsForZoom(selectedZoomLevel);
        expect(result.value).toEqual({ hour: 1, minute: 0 });
    });
});