import _ from 'lodash';
import ScheduleService from "../services/schedule.service";
import ReservationService from "../services/reservation.service";
import useWebSocket from 'react-use-websocket';
import { getStore } from "../services/store.helper";
import UIConstants from "../utils/ui.constants";
import moment from "moment";

const isDebugLoggingOn =false;
function useWeekViewWebSocket(data, setData, summaryItem, setSummaryItem, setShowSummary, startTime, endTime) {

    /**
     * Function to call when websocket is connected
     */
    function onConnect() {
        try {
            if (isDebugLoggingOn) console.log("WS Opened");
            const userDets = localStorage.getItem("user");
            if (userDets) {
                sendMessage(JSON.stringify({ "token": JSON.parse(userDets).websocket_token }));
                if (isDebugLoggingOn)  console.log("Auth token submitted");
            }
        } catch (err) {
            console.log('err', err);
        }
    }


    function fetchBlueprintAndAddToTimeline(id, startTime, endTime) {

        // The websocket message only contains a subset of the details we need, so fetch the full set
        const from = startTime.format(UIConstants.CALENDAR_DATETIME_FORMAT);
        const until = endTime.format(UIConstants.CALENDAR_DATETIME_FORMAT);
        ScheduleService.getTimelineSlimBlueprints(from, until, id)
                       .then((response) => {
                           setData(prevData => ({
                               ...prevData,
                               schedulingUnits: prevData.schedulingUnits.concat(response)
                           }));
                       })
                       .catch(e => console.error("Couldn't retrieve scheduling unit details for id: ", id, e));
    }


    function fetchReservationAndAddToTimeline(id, startTime, endTime) {
        const shouldFetchReservations = getStore(UIConstants.STORE_KEY_TIMELINE).reservationsToggle;
        const from = startTime?.format(UIConstants.CALENDAR_DATETIME_FORMAT);
        const until = endTime?.format(UIConstants.CALENDAR_DATETIME_FORMAT);
        if (shouldFetchReservations) {
            ReservationService.getTimelineReservations(from, until, id)
                              .then((response) => {
                                  setData(prevData => ({
                                  ...prevData,
                                  reservations: prevData.reservations.concat(response)
                                  }));
                              })
                              .catch(e => console.error("Couldn't retrieve reservation details for id: ", id, e));
        }
    }

    function deleteBlueprintFromTimeline(id) {
        const schedulingUnits = data.schedulingUnits
        _.remove(schedulingUnits, function (su) { return su.id === id });
        setData(prevData => ({
            ...prevData,
            schedulingUnits: schedulingUnits
        }))
        if (summaryItem?.id === id) {
            setShowSummary(false);
        }
    }

    function deleteReservationFromTimeline(id) {
        const reservations = data.reservations
        _.remove(reservations, function (res) { return res.id === id });
        setData(prevData => ({
            ...prevData,
            reservations: reservations
        }))
        if (summaryItem?.id === id) {
            setShowSummary(false);
        }
    }


    /**
     * Handles the message received through websocket
     * @param {String} data - String of JSON data
     */
    function handleData(event) {
        const jsonData = JSON.parse(event?.data) || {};
        switch (jsonData.object_type) {
            case 'scheduling_unit_blueprint': {
                switch (jsonData.action) {
                    case 'delete': {
                        deleteBlueprintFromTimeline(jsonData.object_details.id);
                        break;
                    }
                    case 'update': {
                        let inScope = true;
                        if (jsonData?.object_details?.process_start_time) {
                            const unit_start_time = moment(jsonData.object_details.process_start_time, moment.ISO_8601);
                            const unit_stop_time = moment(jsonData.object_details.process_stop_time, moment.ISO_8601);
                            if ( (unit_stop_time < startTime) || (unit_start_time > endTime) ) {
                                inScope = false;
                            }
                        }
                        if (data.schedulingUnits.some(unit => unit.id === jsonData.object_details.id)) {
                            // usually we already have most details, so only update the relevant ones.
                            if (inScope === true) {
                                setData(prevData => ({
                                    ...prevData,
                                    schedulingUnits: prevData.schedulingUnits.map(
                                        unit => unit.id === jsonData.object_details.id ? { ...unit, ...jsonData.object_details } : unit
                                    )
                                }));
                            } else {
                                // object has been moved out of scope, so let's get rid of it
                                deleteBlueprintFromTimeline(jsonData.object_details.id);
                            }
                        } else if (inScope === true) {
                            // ...but sometimes we don't have the details yet, e.g. because it at least used to be
                            // outside the timelines time range, and we need to fetch all details so we don't miss
                            // anything that e.g. got moved into scope.
                            fetchBlueprintAndAddToTimeline(jsonData.object_details.id, startTime, endTime);
                        }

                        if (summaryItem?.id === jsonData.object_details.id) {
                            // Note: we could also update details directly so we don't need to fetch again.
                            // However, we would have to send full deltas via websocket instead of the subset that is
                            // relevant for the timeline (like id, timestamps, and status).
                            // e.g.
                            //
                            // setSummarySettings(prevState => ({
                            //    ...prevState,
                            //    schedulingUnitItem: {...prevState.schedulingUnitItem, ...jsonData.object_details}
                            // }));
                            //
                            // Instead, trigger a full refresh of the details panel:
                            setSummaryItem({ id: jsonData.object_details.id, type: "SCHEDULE" });
                        }
                        break;
                    }
                    case 'create': {
                        // The websocket message only contains a subset of the details we need, so fetch the full set
                        fetchBlueprintAndAddToTimeline(jsonData.object_details.id, startTime, endTime);
                        break;
                    }
                    default: { break; }
                }
                break;
            }
            case 'reservation': {
                switch (jsonData.action) {
                    case 'delete': {
                        deleteReservationFromTimeline(jsonData.object_details.id);
                        break;
                    }
                    case 'update': {
                        let inScope = true;
                        if (jsonData.object_details.start_time) {
                            const res_start_time = moment(jsonData.object_details.start_time, moment.ISO_8601);
                            const res_stop_time = moment(jsonData.object_details.stop_time, moment.ISO_8601);
                            if ( (res_stop_time < startTime) || (res_start_time > endTime) ) {
                                inScope = false;
                            }
                        }
                        if (data.reservations.some(res => res.id === jsonData.object_details.id)) {
                            if (inScope === true) {
                                // usually we already have most details, so only update the relevant ones.
                                setData(prevData => ({
                                    ...prevData,
                                    reservations: prevData.reservations.map(
                                        res => res.id === jsonData.object_details.id ? { ...res, ...jsonData.object_details } : res
                                    ),
                                }));
                            } else {
                                // object has been moved out of scope, so let's get rid of it
                                deleteReservationFromTimeline(jsonData.object_details.id);
                            }
                        } else if (inScope === true) {
                            // ...but sometimes we don't have the details yet, e.g. because it at least used to be
                            // outside the timelines time range, and we need to fetch all details so we don't miss
                            // anything that e.g. got moved into scope.
                            fetchReservationAndAddToTimeline(jsonData.object_details.id, startTime, endTime);
                        }
                        if (summaryItem?.id === jsonData.object_details.id) {
                            // Trigger a full refresh of the details panel
                            setSummaryItem({ id: jsonData.object_details.id, type: "RESERVATION" });
                        }
                        break;
                    }
                    case 'create': {
                        // The websocket message only contains a subset of the details we need, so fetch the full set
                        fetchReservationAndAddToTimeline(jsonData.object_details.id, startTime, endTime);
                        break;
                    }
                    default: { break; }
                }
                break;
            }
            default: { break; }
        }
    }

    // websocket hook that opens and allows interaction via the wss connection
    const {
        sendMessage
    } = useWebSocket(process.env.REACT_APP_WEBSOCKET_URL, {
        onOpen: () => onConnect(),
        onMessage: (event) => handleData(event),
        onError: (event) => { console.error(event); },
        shouldReconnect: () => true,
    });

}

export default useWeekViewWebSocket;